package com.asc.utils;

import java.util.Vector;
import java.util.StringTokenizer;
import java.applet.Applet;

/**
 * Provides extra ways to access applet parameters. Allows multiple values for a
 * parameter either by separated values, or by adding an index value to each
 * parameter (i.e. param0=first param1=second, etc.) You can either call static
 * methods or create an instance and call instance methods. The static methods
 * require that you pass the applet each time. Because you create instances by
 * passing the Applet to the constructor, the instance methods don't require you
 * to pass the applet.
 * 
 * @author Mark Wutka
 */

public class Parameters {
	protected Applet applet;

	/**
	 * Creates a Parameters instance that will fetch parameters for a particular
	 * applet.
	 * 
	 * @param applet
	 *            The applet whose parameters will be retrieved.
	 */

	public Parameters(Applet applet) {
		this.applet = applet;
	}

	// All the instance methods just pass through to the static methods
	// to avoid code redundancy.

	/**
	 * Returns the named parameter, or null if not set (this is identical to the
	 * Applet.getParameter method).
	 * 
	 * @param paramName
	 *            The name of the parameter to retrieve.
	 */
	public String getParameter(String paramName) {
		return getParameter(paramName);
	}

	/**
	 * Returns the named parameter, or defaultValue if not set.
	 * 
	 * @param paramName
	 *            The name of the parameter to retrieve.
	 * @param defaultValue
	 *            The parameter value to use if there is no PARAM tag for this
	 *            parameter.
	 */
	public String getParameter(String paramName, String defaultValue) {
		return getParameter(applet, paramName, defaultValue);
	}

	/**
	 * Returns an array of parameters. The corresponding parameter names in the
	 * PARAM tags should end with numbers starting with 0. For example, for a
	 * parameter named "foo", the PARAM tags should use names of foo0, foo1,
	 * foo2, etc:
	 * 
	 * <PRE>
	 * 
	 * <PARAM name="foo0" value="somevalue"> <PARAM name="foo1"
	 * value="anothervalue"> <PARAM name="foo2" value="thisvalue"> <PARAM
	 * name="foo3" value="lastvalue">
	 * 
	 * </PRE>
	 * 
	 * If you skip a number, the rest of the parameters will be ignored.
	 * 
	 * @param paramName
	 *            The base name for the parameters to be fetched.
	 */

	public String[] getParameters(String paramName) {
		return getParameters(applet, paramName);
	}

	/**
	 * Returns an array of parameters. The parameters should be separated by a
	 * specific separator character, or a set of separator characters. For
	 * example, for the following call to getParameters:
	 * 
	 * <PRE>
	 * 
	 * param.getParameters("foo", ";:,");
	 * 
	 * </PRE>
	 * 
	 * You could use :, ;, and , as separators:
	 * 
	 * <PRE>
	 * 
	 * <PARAM name="foo" value="somevalue:anothervalue;thisvalue,lastvalue">
	 * 
	 * </PRE>
	 * 
	 * @param paramName
	 *            The name of the parameter to fetch.
	 * @param separators
	 *            A string containing the separators for the parameters.
	 */
	public String[] getParameters(String paramName, String separators) {
		return getParameters(applet, paramName, separators);
	}

	/**
	 * Returns the named parameter, or null if not set (this is identical to the
	 * Applet.getParameter method).
	 * 
	 * @param applet
	 *            The applet whose parameters will be fetched.
	 * @param paramName
	 *            The name of the parameter to retrieve.
	 */
	public static String getParameter(Applet applet, String paramName) {
		return applet.getParameter(paramName);
	}

	/**
	 * Returns the named parameter, or defaultValue if not set.
	 * 
	 * @param applet
	 *            The applet whose parameters will be fetched.
	 * @param paramName
	 *            The name of the parameter to retrieve.
	 * @param defaultValue
	 *            The parameter value to use if there is no PARAM tag for this
	 *            parameter.
	 */
	public static String getParameter(Applet applet, String paramName,
			String defaultValue) {
		String returnValue = applet.getParameter(paramName);
		if (returnValue == null)
			return defaultValue;
		return returnValue;
	}

	/**
	 * Returns an array of parameters. The corresponding parameter names in the
	 * PARAM tags should end with numbers starting with 0. For example, for a
	 * parameter named "foo", the PARAM tags should use names of foo0, foo1,
	 * foo2, etc:
	 * 
	 * <PRE>
	 * 
	 * <PARAM name="foo0" value="somevalue"> <PARAM name="foo1"
	 * value="anothervalue"> <PARAM name="foo2" value="thisvalue"> <PARAM
	 * name="foo3" value="lastvalue">
	 * 
	 * </PRE>
	 * 
	 * If you skip a number, the rest of the parameters will be ignored.
	 * 
	 * @param applet
	 *            The applet whose parameters will be fetched.
	 * @param paramName
	 *            The base name for the parameters to be fetched.
	 */
	public static String[] getParameters(Applet applet, String paramName) {
		// Put the parameters into a vector first, because you don't
		// know how many parameters you are getting.
		Vector vec = new Vector();

		for (int i = 0; true; i++) {
			// Try getting next numbered parameter
			String paramStr = applet.getParameter(paramName + i);
			// If it isn't there, you're done
			if (paramStr == null)
				break;
			// If you got the parameter, add the it to the vector
			vec.addElement(paramStr);
		}

		// Create a string array to hold the values
		String[] returnValues = new String[vec.size()];

		// Copy the vector values into the new string array
		vec.copyInto((Object[]) returnValues);

		return returnValues;
	}

	/**
	 * Returns an array of parameters. The parameters should be separated by a
	 * specific separator character or a set of separator characters. For
	 * example, for the following call to getParameters:
	 * 
	 * <PRE>
	 * 
	 * param.getParameters("foo", ";:,");
	 * 
	 * </PRE>
	 * 
	 * You could use :, ;, and , as separators
	 * 
	 * <PRE>
	 * 
	 * <PARAM name="foo" value="somevalue:anothervalue;thisvalue,lastvalue">
	 * 
	 * </PRE>
	 * 
	 * @param paramName
	 *            The name of the parameter to fetch.
	 * @param separators
	 *            A string containing the separators for the parameters.
	 */
	public static String[] getParameters(Applet applet, String paramName,
			String separators) {
		String paramStr = applet.getParameter(paramName);

		// If the parameters weren't there, just return an empty array
		if (paramStr == null) {
			return new String[0];
		}

		// Put the parameters into a vector first, because you don't
		// know how many parameters you are getting.
		Vector vec = new Vector();

		// The tokenizer will separate out the parameters from the string
		StringTokenizer tok = new StringTokenizer(paramStr, separators);

		// Grab the parameters from the string
		while (tok.hasMoreTokens()) {
			vec.addElement(tok.nextToken());
		}

		// Create a string array to hold the values
		String[] returnValues = new String[vec.size()];
		// Copy the vector values into the new string array
		vec.copyInto((Object[]) returnValues);

		return returnValues;
	}
}
